@@ -242,7 +242,7 @@ class Insight_Definition:
         self.components = r.json()["components"]
         return self.components[position]["id"]
 
-    def add_table(self, query="", columns=[]):
+    def add_table(self, query="", columns=[], pivot_col=[], pivot_type="2", max_results=50):
 
         position = (len(self.components))
         body = {"type": "table", "position": position}
@@ -252,11 +252,11 @@ class Insight_Definition:
         self.components = r.json()["components"]
 
         if(query != "" and columns!=[]):
-            self.edit_table(self.components[position]["id"], query, columns)
+            self.edit_table(self.components[position]["id"], query, columns, pivot_col, pivot_type, max_results)
 
         return self.components[position]["id"]
 
-    def edit_table(self, table_id, query="", columns=[], pivot_col=[], pivot_type="2"):
+    def edit_table(self, table_id, query="", columns=[], pivot_col=[], pivot_type="2", max_results=50):
 
         url = self.smart_insight_url + "/definitions/" + self.definition_id + "/tables/" + table_id
 
@@ -265,19 +265,51 @@ class Insight_Definition:
             "type": "table",
             "name": "Table",
             "customName":False,
-            "dependsOn": ["q1"],
+            "dependsOn": query,
             "queryId": query,
             "columns": columns,
-            "maxResults": 50,
+            "maxResults": max_results,
             "valid": True,
             "queryComponent":False,
-            "pivot": len(pivot_col)!=0,
+            "pivot": True,
             "pivotColumns": pivot_col,
             "pivotType": pivot_type
         }
+
         r = self.__make_put_request(url, body)
         self.components = r.json()["components"]
 
+    def pivot_table(self, table_id, query="", columns=[], pivot_col=[], max_results=50):
+
+        url = self.smart_insight_url + "/definitions/" + self.definition_id + "/tables/" + table_id
+
+        body = {
+            "id": table_id,
+            "type": "table",
+            "name": "Table",
+            "customName": False,
+            "dependsOn": [query],
+            "queryId": query,
+            "columns": columns,
+            "maxResults": max_results,
+            "valid": True,
+            "queryComponent": False,
+            "pivot": True,
+            "pivotColumns": pivot_col,
+            "pivotType": "2"
+        }
+
+        s = requests.Session()
+        s.keep_alive = False
+        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+        s.mount('https://', HTTPAdapter(max_retries=retries))
+        headers = {
+            "Content-Type": "application/json",
+            "Authorization": "Bearer" + " " + self._token
+        }
+        r = s.put(url=url, json=body, headers=headers)
+        r.raise_for_status()
+        self.components = r.json()["components"]
 
     def get_result_set(self, query_id):
 
@@ -323,52 +355,32 @@ class Insight_Definition:
 
     def edit_chart(self, chart_id, chart_type:str, query:str, params, title = "Chart"):
 
-        '''url = self._base_url_askdata+"/smartinsight/definitions/"+self.definition_id+"/queries/"+query.lower()+"/columns"
+        if(not query.startswith("qcomp")):
 
-        s = requests.Session()
-        s.keep_alive = False
-        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
-        s.mount('https://', HTTPAdapter(max_retries=retries))
+            url = self._base_url_askdata+"/smartinsight/definitions/"+self.definition_id+"/queries/"+query.lower()+"/columns"
+            r = self.__make_get_request(url)
+            columns = r.json()
 
-        headers = {
-            "Content-Type": "application/json",
-            "Authorization": "Bearer" + " " + self._token
-        }
-        r = s.get(url=url, headers=headers)
-        r.raise_for_status()
-        columns = r.json()
+            while(columns == []):
 
-        while(columns == []):
-            url = self._base_url_askdata + "/smartinsight/definitions/" + self.definition_id + "/queries/" + query.lower() + "/columns"
+                r = self.__make_get_request(url)
+                columns = r.json()
+
+        else:
+            qc = self.get_component(query)
+
+            url_preview = self.smart_insight_url+"/composed_queries/"+qc["settings"]["qc"]+"/preview?limit=100&agentId="+self.agent_id
 
             s = requests.Session()
             s.keep_alive = False
             retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
             s.mount('https://', HTTPAdapter(max_retries=retries))
 
-            headers = {
-                "Content-Type": "application/json",
-                "Authorization": "Bearer" + " " + self._token
-            }
-            r = s.get(url=url, headers=headers)
-            r.raise_for_status()
-            columns = r.json()
-        '''
-
-        qc = self.get_component(query)
-
-        url_preview = self.smart_insight_url+"/composed_queries/"+qc["settings"]["qc"]+"/preview?limit=100&agentId="+self.agent_id
-
-        s = requests.Session()
-        s.keep_alive = False
-        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
-        s.mount('https://', HTTPAdapter(max_retries=retries))
-
-        headers = {"Authorization": "Bearer" + " " + self._token}
-        response = s.post(url=url_preview, json={}, headers=headers)
-        response.raise_for_status()
-        r = response.json()
-        columns = r["schema"]
+            headers = {"Authorization": "Bearer" + " " + self._token}
+            response = s.post(url=url_preview, json={}, headers=headers)
+            response.raise_for_status()
+            r = response.json()
+            columns = r["schema"]
 
         upper_params = []
         for param in params:
